@@ -1,6 +1,6 @@
 package DBD::Gofer::Transport::stream;
 
-#   $Id: stream.pm 9395 2007-04-11 15:57:46Z timbo $
+#   $Id: stream.pm 10087 2007-10-16 12:42:37Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -14,7 +14,7 @@ use Carp;
 
 use base qw(DBD::Gofer::Transport::pipeone);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 9395 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 10087 $ =~ /(\d+)/o);
 
 __PACKAGE__->mk_accessors(qw(
     go_persist
@@ -148,7 +148,7 @@ sub receive_response_by_transport {
     my ($pid, $rfh, $efh, $cmd) = @{$connection}{qw(pid rfh efh cmd)};
 
     my $errno = 0;
-    my $frozen_response;
+    my $encoded_response;
     my $stderr_msg;
 
     $self->read_response_from_fh( {
@@ -160,14 +160,14 @@ sub receive_response_by_transport {
         $rfh => {
             error => sub { warn "error reading response: $!"; $errno||=$!; 1 },
             eof   => sub { warn "eof on stdout" if 0; 1 },
-            read  => sub { $frozen_response .= $_; ($frozen_response=~s/\015\012$//) ? 1 : 0 },
+            read  => sub { $encoded_response .= $_; ($encoded_response=~s/\015\012$//) ? 1 : 0 },
         },
     });
 
     # if we got no output on stdout at all then the command has
     # probably exited, possibly with an error to stderr.
     # Turn this situation into a reasonably useful DBI error.
-    if (not $frozen_response) {
+    if (not $encoded_response) {
         my $msg = "No response received";
         if (chomp $stderr_msg && $stderr_msg) {
             $msg .= sprintf ", error reported by \"%s\" (pid %d%s): %s",
@@ -181,14 +181,16 @@ sub receive_response_by_transport {
         die "$msg\n";
     }
 
-    $self->trace_msg("Response received: $frozen_response\n",0)
+    $self->trace_msg("Response received: $encoded_response\n",0)
         if $trace >= 4;
 
     $self->trace_msg("Gofer stream stderr message: $stderr_msg\n",0)
         if $stderr_msg && $trace;
 
+    my $frozen_response = pack("H*", $encoded_response);
+
     # XXX need to be able to detect and deal with corruption
-    my $response = $self->thaw_response(pack("H*",$frozen_response));
+    my $response = $self->thaw_response($frozen_response);
 
     if ($stderr_msg) {
         # add stderr messages as warnings (for PrintWarn)
@@ -211,7 +213,7 @@ sub transport_timedout {
 __END__
 
 =head1 NAME
-    
+
 DBD::Gofer::Transport::stream - DBD::Gofer transport for stdio streaming
 
 =head1 SYNOPSIS
@@ -270,17 +272,21 @@ Automatically reconnect (within reason) if there's a transport error.
 
 Decide on default for persistent connection - on or off? limits? ttl?
 
-=head1 AUTHOR AND COPYRIGHT
+=head1 AUTHOR
 
-The DBD::Gofer, DBD::Gofer::* and DBI::Gofer::* modules are
-Copyright (c) 2007 Tim Bunce. Ireland.  All rights reserved.
+Tim Bunce, L<http://www.tim.bunce.name>
 
-You may distribute under the terms of either the GNU General Public License or
-the Artistic License, as specified in the Perl README file.
+=head1 LICENCE AND COPYRIGHT
+
+Copyright (c) 2007, Tim Bunce, Ireland. All rights reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself. See L<perlartistic>.
 
 =head1 SEE ALSO
 
+L<DBD::Gofer::Transport::Base>
+
 L<DBD::Gofer>
 
 =cut
-
